@@ -7,7 +7,7 @@ Graph Matching
 Given a graph G = (V,E), a matching M in G is a set of pairwise non-adjacent
 edges; that is, no two edges share a common vertex.
 
-http://en.wikipedia.org/wiki/Matching_(graph_theory)
+`Wikipedia: Matching <https://en.wikipedia.org/wiki/Matching_(graph_theory)>`_
 """
 #   Copyright (C) 2011-2012 by
 #   Nicholas Mancuso <nick.mancuso@gmail.com>
@@ -17,6 +17,7 @@ import networkx as nx
 __all__ = ["min_maximal_matching"]
 __author__ = """Nicholas Mancuso (nick.mancuso@gmail.com)"""
 
+
 def min_maximal_matching(G):
     r"""Returns the minimum maximal matching of G. That is, out of all maximal
     matchings of the graph G, the smallest is returned.
@@ -37,7 +38,7 @@ def min_maximal_matching(G):
     -----
     The algorithm computes an approximate solution fo the minimum maximal
     cardinality matching problem. The solution is no more than 2 * OPT in size.
-    Runtime is `O(|E|)`.
+    Runtime is $O(|E|)$.
 
     References
     ----------
