@@ -7,18 +7,18 @@ Do not add this file to the repository.
 
 import datetime
 
-version = '1.11'
-date = 'Sat Jan 30 09:55:40 2016'
+version = '2.2'
+date = 'Wed Sep 19 18:49:08 2018'
 
 # Was NetworkX built from a development version? If so, remember that the major
 # and minor versions reference the "target" (rather than "current") release.
 dev = False
 
 # Format: (name, major, min, revision)
-version_info = ('networkx', '1', '11', None)
+version_info = ('networkx', '2', '2', None)
 
 # Format: a 'datetime.datetime' instance
-date_info = datetime.datetime(2016, 1, 30, 9, 55, 40, 165755)
+date_info = datetime.datetime(2018, 9, 19, 18, 49, 8)
 
 # Format: (vcs, vcs_tuple)
 vcs_info = (None, (None, None))
